@@ -2,7 +2,10 @@ import numpy as np
 import pytest
 
 import pandas as pd
-from pandas.core.arrays.floating import Float32Dtype, Float64Dtype
+from pandas.core.arrays.floating import (
+    Float32Dtype,
+    Float64Dtype,
+)
 
 
 def test_dtypes(dtype):
