@@ -33,7 +33,7 @@ _default_compressor = "blosc"
 pytestmark = pytest.mark.single_cpu
 
 
-def test_conv_read_write(setup_path):
+def test_conv_read_write():
     with tm.ensure_clean() as path:
 
         def roundtrip(key, obj, **kwargs):
@@ -491,20 +491,6 @@ def _check_roundtrip(obj, comparator, path, compression=False, **kwargs):
         comparator(retrieved, obj, **kwargs)
 
 
-def _check_double_roundtrip(self, obj, comparator, path, compression=False, **kwargs):
-    options = {}
-    if compression:
-        options["complib"] = compression or _default_compressor
-
-    with ensure_clean_store(path, "w", **options) as store:
-        store["obj"] = obj
-        retrieved = store["obj"]
-        comparator(retrieved, obj, **kwargs)
-        store["obj"] = retrieved
-        again = store["obj"]
-        comparator(again, obj, **kwargs)
-
-
 def _check_roundtrip_table(obj, comparator, path, compression=False):
     options = {}
     if compression:
